# =============================================================================
# <RC_analyzer to automatically evaluate PET/CT/MR gantry aligment>
#     Copyright (C) <2021>  <Jaroslav Ptacek (WG PET/CT/MRI QC EFOMP)>
# 
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <https://www.gnu.org/licenses/>.
# =============================================================================

import os
from win32api import GetSystemMetrics

import pydicom as dcm

import numpy as np
import pandas as pd
import math

import tkinter as tk
import tkinter.filedialog as filedialog

from matplotlib import cm

from PIL import Image, ImageTk

# =============================================================================
# Data
# =============================================================================
class phantomData:
    def __init__(self, directory, data_PT, data_CT, data_PT_thresh, data_CT_thresh, all_setup):
# =============================================================================
# object properties
# =============================================================================
        self.directory = directory
        self.PT = data_PT
        self.CT = data_CT
        self.PT_thresh = data_PT_thresh
        self.CT_thresh = data_CT_thresh
        self.primary_zoom = []
        self.all_setup = all_setup

        self.zoom = {'PT':1, 'CT':1}
        self.slice = 0

        self.position = {'PT':[0,0], 'CT':[0,0]} # phantom image display position [row x col]

        self.z_position_PT = []
        self.z_position_CT = []

        self.intersection_PT = []
        self.intersection_CT = []

        self.image_to_show = {'PT':dict(), 'CT':dict()}

        self.diffs = []

# =============================================================================
# object functions
# =============================================================================
    def show_me_projections(self, of_what, modality, where, position, zoom):

        for i in [0, 1, 2]:
            temp = np.max(of_what[:,:,:],i)
            temp = Image.fromarray(cm.gray(temp, bytes=True))
            temp = temp.resize((int(where[i].winfo_width()*zoom[modality]), int(where[i].winfo_height()*zoom[modality]))) # zoom according to window size col x rpw
            self.primary_zoom = [self.all_setup[modality]['rows']/where[i].winfo_height(), self.all_setup[modality]['rows']/where[i].winfo_width()] # size row x col
            self.image_to_show[modality][i] = ImageTk.PhotoImage(temp)
            where[i].create_image(position[modality][0],position[modality][1], anchor='nw', image=self.image_to_show[modality][i]) # this is col x row

    def slice_zoom(self, event, what, zoom_point, modality, where):
    # zoom[0] scroll button rotation, zoom[1] position rows, zoom[2] position cols, where = where to zoom
        zoom_step = 0.5
        if (zoom_point[0] > 0):
            if self.zoom[modality] >= 5: # zoom setup = property
                ...
            else:
                self.zoom[modality] += (120/zoom_point[0])*zoom_step
                self.position[modality] = [self.position[modality][0]-zoom_point[1]*zoom_step, self.position[modality][1]-zoom_point[2]*zoom_step] # position setup = property
                self.show_me_projections(what, modality, where, self.position, self.zoom)
        elif (zoom_point[0] < 0):
            if self.zoom[modality] <= 1:
                ...
            else:
                self.position[modality] = [self.position[modality][0]+zoom_point[1]*zoom_step, self.position[modality][1]+zoom_point[2]*zoom_step]
                self.zoom[modality] += (120/zoom_point[0])*zoom_step
                self.show_me_projections(what, modality, where, self.position, self.zoom)

    # centre of mass calculation
    def cm_calculation(self, masked_array, row_list, col_list):

        total_mass = np.sum(masked_array)

        if total_mass == 0:
            total_mass = np.finfo(float).eps

        temp_col = col_list*np.sum(masked_array,0)
        temp_row = row_list*np.sum(masked_array,1)

        cm_row = float((sum(temp_row)/total_mass))
        cm_col = float((sum(temp_col)/total_mass))

        return (cm_row, cm_col)
    # indentification and location of the line
    def line_finder(self, where, masking, sources, skipper):
        masking = masking # masking in px size
        sources = sources # number of sources
        skipper = skipper # calculation based on each x-th slice - interconnected with masking - larger skipper value demands larger masking value to work

        position = []
        position_temp = []

        temp = where[:,:,0]
        mask = np.ones((np.size(temp,0),np.size(temp,1)))

        # locate max positions on the first slice
        for s in list(range(0,sources)): # number of sources = number of max positions

            ind = np.where(mask*temp == np.amax(mask*temp))
            ind = [ind[0][0], ind[1][0]]

            position_temp.extend(ind)
            mask[ind[0]-masking:ind[0]+masking, ind[1]-masking:ind[1]+masking] = 0
        position.append(position_temp)

        # calculate centre of mass of the identified line
        rows = list(range(0, np.size(temp,0)))
        columns = list(range(0, np.size(temp,1)))

        cm_position = []
        cm_position_temp = []
        mask = np.zeros((np.size(temp,0),np.size(temp,1)))

        slices = np.size(where,2)

        for i in range(0, slices, skipper):
            temp = where[:,:,i]
            position_temp = []
            cm_position_temp = []

            for s in list(range(0,sources)):
                # position[-1] takes the preceding element
                row_prev = position[-1][2*s]
                col_prev = position[-1][2*s+1]

                mask[row_prev-masking:row_prev+masking, col_prev-masking:col_prev+masking] = 1
                ind = np.where(mask*temp == np.amax(mask*temp))
                ind = [ind[0][0], ind[1][0]]

                position_temp.extend(ind)

                cm_position_temp.extend(self.cm_calculation(mask*temp, rows, columns))
                mask[row_prev-masking:row_prev+masking, col_prev-masking:col_prev+masking] = 0

            position.append(position_temp)
            cm_position.append(cm_position_temp)
        return cm_position

    def calculate_intersections(self, modality_pos_data, modality, skipper):
        # locate the intersection of two line sources

        img_plane = [0,1] # coronal 0 a sagital 1 image plane - must go through both - orientation of the phantom unknown

        for p in img_plane:
            x1 = []
            x2 = []
            z = []

            if modality == 'PT':
                z = self.z_position_PT[::skipper]

            if modality == 'CT':
                z = self.z_position_CT[::skipper]

            for i in modality_pos_data:
                # first and third column - image plane 0 
                # second and fourth column - image plane 1
                x1.append(i[p])
                x2.append(i[p+2])

            # linear equation of lines
            model1 = np.linalg.lstsq(np.vstack([x1, np.ones(len(x1))]).T,np.array(z), rcond=None)[0] # least square fit
            model2 = np.linalg.lstsq(np.vstack([x2, np.ones(len(x2))]).T,np.array(z), rcond=None)[0]

            z1 = []
            z2 = []
            for x in x1:
                z1.append(model1[0]*x+model1[1])

            for x in x2:
                z2.append(model2[0]*x+model2[1])

            x_int = (model2[1]-model1[1])/(model1[0]-model2[0])
            z_int1 = model1[0]*x_int+model1[1] # intersection coordinates
            z_int2 = model2[0]*x_int+model2[1]

            if modality == 'PT':
                if abs(z_int1) < np.max(np.abs(self.z_position_PT))  and abs(z_int2) < np.max(np.abs(self.z_position_PT)) \
                    and abs(z_int1) > np.min(np.abs(self.z_position_PT)) and abs(z_int2) > np.min(np.abs(self.z_position_PT)): # if the result is inside the FOV - OK
                    result = z_int1, z_int2, model1, model2 # not multipled byt the slice thickness, already included in image_position_patient[2]
                    return(result)
                else:
                    ...

            if modality == 'CT':
                if abs(z_int1) < np.max(np.abs(self.z_position_CT))  and abs(z_int2) < np.max(np.abs(self.z_position_CT)) \
                    and abs(z_int1) > np.min(np.abs(self.z_position_CT)) and abs(z_int2) > np.min(np.abs(self.z_position_CT)): # if the result is inside the FOV - OK
                    result = z_int1, z_int2, model1, model2 # not multipled byt the slice thckness, already included in image_position_patient[2]

                    return(result)
                else:
                    ...

    def get_out_results(self, file_path):
        temp = []

        if len(self.diffs) == 0:
            tk.messagebox.showwarning("Warning", 'Proceed with the calculation first!')
            return

        # slice position related to x,y coordinates 
        for i in range(0, len(self.pos_PT_mm)):
            temp.append(self.pos_PT_mm[i]+[self.z_position_PT[i]]+self.pos_CT_mm[i]+[self.z_position_CT[i]])

        columns = ['x_PT1', 'y_PT1', 'x_PT2', 'y_PT2', 'z_PT', 'x_CT1', 'y_CT1', 'x_CT2', 'y_CT2', 'z_CT']

        temp = pd.DataFrame(temp, columns = columns)

        temp['PT1_slope'] = self.intersection_PT[2][0]
        temp['PT1_intercept'] = self.intersection_PT[2][1]
        temp['PT2_slope'] = self.intersection_PT[3][0]
        temp['PT2_intercept'] = self.intersection_PT[3][0]
        temp['CT1_slope'] = self.intersection_CT[2][0]
        temp['CT1_intercept'] = self.intersection_CT[2][1]
        temp['CT2_slope'] = self.intersection_CT[3][0]
        temp['CT2_intercept'] = self.intersection_CT[3][1]

        file_name = self.all_setup['PT']['device'] + '_' + self.all_setup['PT']['study_date'][0:2] + self.all_setup['PT']['study_date'][3:5] + \
        self.all_setup['PT']['study_date'][6:10] + 'result.csv'
        file_name = file_name.replace(" ", "") # delete spaces
        file_name = file_name.replace("/", "_") # delete backslash

        file_dir = filedialog.asksaveasfilename(parent=root, initialfile=file_name, initialdir=file_path ,title='Please select a directory to save') # find the directory with data

        if file_dir: # check - file name entered
            pd.DataFrame(temp).to_csv(file_dir, sep=';')
            tk.messagebox.showinfo("Export", "Export finished!")
        else:
            ...

# =============================================================================
# main window
# =============================================================================
class mainWindow:
    def __init__(self, master):
        self.master = master

        # self.master.geometry('+1915+0')
        self.master.title("Alignment analyzer")
        self.master.resizable(0,0)

        self.Xpix = GetSystemMetrics(0)
        self.Ypix = GetSystemMetrics(1)
        self.Xsize = (self.Xpix-350)/6
        self.Ysize = self.Xsize

        self.plus = tk.PhotoImage(file = r"plus.png")
        self.minus = tk.PhotoImage(file = r"minus.png")

        self.threshold = {'PT': tk.StringVar(), 'CT': tk.StringVar()}
        self.threshold['PT'].set('5')
        self.threshold['CT'].set('25')

        self.masking = tk.StringVar() # initital masking value in px
        self.masking.set('15')

        self.sources = tk.StringVar() # initial number of sources
        self.sources.set('2')

        self.skipper = tk.StringVar() # initial skipper value - closely connected to masking - larger skipper = larger masking
        self.skipper.set('10')

        self.CE = tk.StringVar()
        self.CE.set('???')

        self.CE_xshift = tk.StringVar()
        self.CE_xshift.set('???')

        self.CE_yshift = tk.StringVar()
        self.CE_yshift.set('???')

        self.xshift_max = tk.StringVar()
        self.xshift_max.set('???')

        self.xshift_mean = tk.StringVar()
        self.xshift_mean.set('???')

        self.yshift_max = tk.StringVar()
        self.yshift_max.set('???')

        self.yshift_mean = tk.StringVar()
        self.yshift_mean.set('???')

        self.zshift = tk.StringVar()
        self.zshift.set('???')

        self.frame_a = tk.LabelFrame(master, text='Data PT')
        self.frame_b = tk.LabelFrame(master, text='Data CT')
        self.frame_c = tk.LabelFrame(master, text='Data preparation and analysis')
        self.frame_d = tk.LabelFrame(master, text='Load data, DICOM info')
        self.frame_e = tk.LabelFrame(master, text='Results - shift between PET and CT image')

        # =============================================================================
        # frame_a
        # =============================================================================
        self.phantom_PT_trans = tk.Canvas(self.frame_a, width=self.Xsize, height=self.Ysize, borderwidth=2, bg='white')
        self.phantom_PT_trans.config(highlightbackground='black', highlightthickness='2')
        self.phantom_PT_trans.grid(row=0, column=0)

        self.phantom_PT_sag = tk.Canvas(self.frame_a, width=self.Xsize, height=self.Ysize, borderwidth=2, bg='white')
        self.phantom_PT_sag.config(highlightbackground='black', highlightthickness='2')
        self.phantom_PT_sag.grid(row=0, column=1)

        self.phantom_PT_cor = tk.Canvas(self.frame_a, width=self.Xsize, height=self.Ysize, borderwidth=2, bg='white')
        self.phantom_PT_cor.config(highlightbackground='black', highlightthickness='2')
        self.phantom_PT_cor.grid(row=0, column=2)

        label = tk.Label(self.frame_a, text='transverzal', anchor='nw', justify='left').grid(row=1, column=0)
        label = tk.Label(self.frame_a, text='sagital', anchor='nw', justify='left').grid(row=1, column=1)
        label = tk.Label(self.frame_a, text='coronal', anchor='nw', justify='left').grid(row=1, column=2)

        # =============================================================================
        # frame_b
        # =============================================================================
        self.phantom_CT_trans = tk.Canvas(self.frame_b, width=self.Xsize, height=self.Ysize, borderwidth=2, bg='white')
        self.phantom_CT_trans.config(highlightbackground='black', highlightthickness='2')
        self.phantom_CT_trans.grid(row=0, column=0)

        self.phantom_CT_sag = tk.Canvas(self.frame_b, width=self.Xsize, height=self.Ysize, borderwidth=2, bg='white')
        self.phantom_CT_sag.config(highlightbackground='black', highlightthickness='2')
        self.phantom_CT_sag.grid(row=0, column=1)

        self.phantom_CT_cor = tk.Canvas(self.frame_b, width=self.Xsize, height=self.Ysize, borderwidth=2, bg='white')
        self.phantom_CT_cor.config(highlightbackground='black', highlightthickness='2')
        self.phantom_CT_cor.grid(row=0, column=2)

        label = tk.Label(self.frame_b, text='transverzal', anchor='nw', justify='left').grid(row=1, column=0)
        label = tk.Label(self.frame_b, text='sagital', anchor='nw', justify='left').grid(row=1, column=1)
        label = tk.Label(self.frame_b, text='coronal', anchor='nw', justify='left').grid(row=1, column=2)

        # =============================================================================
        # frame_c
        # =============================================================================
        width = 6
        label = tk.Label(self.frame_c, textvariable=self.threshold['PT'], width = width)
        label.grid(row=0, column=0, sticky='nsew')
        label = tk.Label(self.frame_c, text='PT threshold (%)', anchor='sw', justify='left').grid(row=0, column=1, sticky='nsew')

        button = tk.Button(self.frame_c, image = self.minus, width="20", height="20", command = lambda modality = 'PT': self.thresh_modif(-1, modality)).grid(row=0, column=2, sticky='nsew')
        button = tk.Button(self.frame_c, image = self.plus, width="20", height="20", command = lambda modality = 'PT': self.thresh_modif(1, modality)).grid(row=0, column=3, sticky='nsew')

        label = tk.Label(self.frame_c, textvariable=self.threshold['CT'], width = width)
        label.grid(row=1, column=0, sticky='nsew')
        label = tk.Label(self.frame_c, text='CT threshold (%)', anchor='sw', justify='left').grid(row=1, column=1, sticky='nsew')

        button = tk.Button(self.frame_c, image = self.minus, width="20", height="20", command = lambda modality = 'CT': self.thresh_modif(-1, modality)).grid(row=1, column=2, sticky='nsew')
        button = tk.Button(self.frame_c, image = self.plus, width="20", height="20", command = lambda modality = 'CT': self.thresh_modif(+1, modality)).grid(row=1, column=3, sticky='nsew')

        label = tk.Label(self.frame_c, textvariable=self.skipper, width = width)
        label.grid(row=2, column=0, sticky='nsew')
        label = tk.Label(self.frame_c, text='number of skipped slices', anchor='sw', justify='left').grid(row=2, column=1, sticky='nsew')

        button = tk.Button(self.frame_c, image = self.minus, width="20", height="20", command = lambda: self.skipper_modif(-1)).grid(row=2, column=2, sticky='nsew')
        button = tk.Button(self.frame_c, image = self.plus, width="20", height="20", command = lambda: self.skipper_modif(+1)).grid(row=2, column=3, sticky='nsew')

        label = tk.Label(self.frame_c, textvariable=self.masking, width = width)
        label.grid(row=3, column=0, sticky='nsew')
        label = tk.Label(self.frame_c, text='size of the masking element ', anchor='sw', justify='left').grid(row=3, column=1, sticky='nsew')

        button = tk.Button(self.frame_c, image = self.minus, width="20", height="20", command = lambda: self.mask_modif(-1)).grid(row=3, column=2, sticky='nsew')
        button = tk.Button(self.frame_c, image = self.plus, width="20", height="20", command = lambda: self.mask_modif(+1)).grid(row=3, column=3, sticky='nsew')

        label = tk.Label(self.frame_c).grid(row=4, column=0, sticky='nsew')

        button = tk.Button(self.frame_c, text = 'Calculate results', command = self.calculate)
        button.grid(row=5, column=0, columnspan = 4, sticky='nsew')

        label = tk.Label(self.frame_c).grid(row=6, column=0, sticky='nsew')

        button = tk.Button(self.frame_c, text = 'Export results', command = self.get_out_results)
        button.grid(row=7, column=0, columnspan = 4, sticky='nsew')

        # =============================================================================
        # frame_d
        # =============================================================================
        button = tk.Button(self.frame_d, text="Open data", command=self.open_data)
        button.grid(row=1, column=0, sticky='w')
        self.data_size = tk.Label(self.frame_d, text='\n\n\n\n\n\n\n\n\n\n\n', anchor='sw', justify='left')
        self.data_size.grid(row=2, column=0)

        # =============================================================================
        # frame_e
        # =============================================================================
        label = tk.Label(self.frame_e, text="Maximum x,y coregistration error:", anchor='sw', justify='left').grid(row=0, column=0, sticky='nsew')
        label = tk.Label(self.frame_e, textvariable = self.CE).grid(row=0, column=1)
        label = tk.Label(self.frame_e, text=" mm").grid(row=0, column=3)

        label = tk.Label(self.frame_e, text="Corresponding x-shift:", anchor='sw', justify='left').grid(row=1, column=0, sticky='nsew')
        label = tk.Label(self.frame_e, textvariable = self.CE_xshift).grid(row=1, column=1)
        label = tk.Label(self.frame_e, text=" mm").grid(row=1, column=3)

        label = tk.Label(self.frame_e, text="Corresponding y-shift:", anchor='sw', justify='left').grid(row=2, column=0, sticky='nsew')
        label = tk.Label(self.frame_e, textvariable = self.CE_yshift).grid(row=2, column=1)
        label = tk.Label(self.frame_e, text=" mm").grid(row=2, column=3)

        label = tk.Label(self.frame_e).grid(row=3, column=0)

        label = tk.Label(self.frame_e, text="Maximum observed x-shift:", anchor='sw', justify='left').grid(row=4, column=0, sticky='nsew')
        label = tk.Label(self.frame_e, textvariable = self.xshift_max).grid(row=4, column=1)
        label = tk.Label(self.frame_e, text=" mm").grid(row=4, column=3)

        label = tk.Label(self.frame_e, text="Mean observed x-shift:", anchor='sw', justify='left').grid(row=5, column=0, sticky='nsew')
        label = tk.Label(self.frame_e, textvariable = self.xshift_mean).grid(row=5, column=1)
        label = tk.Label(self.frame_e, text=" mm").grid(row=5, column=3)

        label = tk.Label(self.frame_e).grid(row=6, column=0)

        label = tk.Label(self.frame_e, text="Maximum observed y-shift:", anchor='sw', justify='left').grid(row=7, column=0, sticky='nsew')
        label = tk.Label(self.frame_e, textvariable = self.yshift_max).grid(row=7, column=1)
        label = tk.Label(self.frame_e, text=" mm").grid(row=7, column=3)

        label = tk.Label(self.frame_e, text="Mean observed y-shift:", anchor='sw', justify='left').grid(row=8, column=0, sticky='nsew')
        label = tk.Label(self.frame_e, textvariable = self.yshift_mean).grid(row=8, column=1)
        label = tk.Label(self.frame_e, text=" mm").grid(row=8, column=3)

        label = tk.Label(self.frame_e).grid(row=9, column=0)

        label = tk.Label(self.frame_e, text="Calculated z-shift:", anchor='sw', justify='left').grid(row=10, column=0, sticky='nsew')
        label = tk.Label(self.frame_e, textvariable = self.zshift).grid(row=10, column=1)
        label = tk.Label(self.frame_e, text=" mm").grid(row=10, column=3)

        # =============================================================================
        # master.grid
        # =============================================================================
        self.frame_a.grid(row=0, column=0, columnspan=3)
        self.frame_b.grid(row=1, column=0, columnspan=3)
        self.frame_c.grid(row=2, column=1)
        frame_switch(self.frame_c, 0) # disabled until data opened
        self.frame_d.grid(row=2, column=0, columnspan=1, sticky='nsew')
        self.frame_e.grid(row=2, column=2, sticky='nsew')
        frame_switch(self.frame_e, 0) # disabled until data opened

# =============================================================================
# main window functions
# =============================================================================

    def open_data(self):
        dirname = filedialog.askdirectory(parent=root,initialdir="/",title='Please select a directory') # find directory with data

        # if no directory selected
        if not dirname:
            return

        else:
            file_lst = []  # create an empty file list
            for item in os.listdir(dirname): # lists the selected directory - only the selected directory!!!
                if os.path.isfile(os.path.join(dirname,item)):
                    file_lst.append(os.path.join(dirname,item))

            if len(file_lst) % 2 != 0:
                tk.messagebox.showwarning("Warning", 'No DICOM data / PT and CT series are not of an equal length!')
                return
            else:
                temp_data = {} # data from the slice will be stored in the dictionary
                temp_data['PT'] = {} # this is for PT
                temp_data['CT'] = {} # this is for CT slices

                # opens files one after another and stores the data in the dictionary
                for i in range(0, len(file_lst)):

                    # check if the file is a DICOM one
                    try:
                        temp = dcm.read_file(file_lst[i]) # read the reference slice with DICOM header and all relevant information
                        modality = str(temp[0x0008, 0x0060].value)
                        number = int(temp[0x0020, 0x0013].value)
                        position = int(temp[0x0020, 0x0032].value[2]) # slices sorted based on their position

                        if modality == 'PT' and number == 1:
                            # algorithm testing point - shift = 0 ... unmodified images
                            shift = 0 # shift in px ... can be changed to check the correctness of calculations
                            temp_data[modality][position] = np.roll(np.roll(temp.pixel_array*temp.RescaleSlope + temp.RescaleIntercept, shift, axis=0), shift, axis=1)
                        else:
                            temp_data[modality][position] = temp.pixel_array*temp.RescaleSlope + temp.RescaleIntercept

                        # read the reference slice
                        if modality == 'PT' and number == 1:
                            ref_PT = dcm.read_file(file_lst[i], stop_before_pixels=True)

                        elif modality == 'CT' and number == 1:
                            ref_CT = dcm.read_file(file_lst[i], stop_before_pixels=True)
                    except: # if the file is not DICOM it's just skipped
                        ...

            data_info = {}
            data_info['PT'] = {}
            data_info['CT'] = {}

            # data_info as a dictionary
            data_info['PT'] = {'rows':ref_PT[0x0028, 0x0010].value, 'cols':ref_PT[0x0028, 0x0011].value, 'slices':len(temp_data['PT']), \
                          'slice_thickness':str(ref_PT.SliceThickness), \
                          'pixel_size':[str(ref_PT.PixelSpacing[0]), str(ref_PT.PixelSpacing[1])], \
                          'device': ref_PT[0x0008,0x1090].value, \
                          'study_date': ref_PT.StudyDate[6:8]+'.'+ref_PT.StudyDate[4:6]+'.'+ref_PT.StudyDate[0:4], \
                          'convul': ref_PT.StudyDate[6:8]+'.'+ref_PT.StudyDate[4:6]+'.'+ref_PT.StudyDate[0:4], \
                          'study_date': ref_PT.StudyDate[6:8]+'.'+ref_PT.StudyDate[4:6]+'.'+ref_PT.StudyDate[0:4], \
                          'image_position_patient': ref_PT[0x0020, 0x0032].value}

            data_info['CT'] = {'rows':ref_CT[0x0028, 0x0010].value, 'cols':ref_CT[0x0028, 0x0011].value, 'slices':len(temp_data['CT']), \
              'slice_thickness':str(ref_CT.SliceThickness), \
              'pixel_size':[str(ref_CT.PixelSpacing[0]), str(ref_CT.PixelSpacing[1])], \
              'device': ref_CT[0x0008,0x1090].value, \
              'study_date': ref_CT.StudyDate[6:8]+'.'+ref_CT.StudyDate[4:6]+'.'+ref_CT.StudyDate[0:4], \
              'image_position_patient': ref_CT[0x0020, 0x0032].value}

            # item 'text' is stored as a dictionary in the Label named self.data_size (see frame_d)
            self.data_size['text'] ='PET' + '\n' \
                                    'rows: ' + str(data_info['PT']['rows']) + '\n' \
                                    'columns: ' + str(data_info['PT']['cols']) + '\n' \
                                    'slices: ' + str(data_info['PT']['slices']) + '\n' \
                                    'slices thickness: ' + str(data_info['PT']['slice_thickness']) + ' mm' +'\n' \
                                    'pixel size: ' + str(data_info['PT']['pixel_size']) + ' mm' +'\n' \
                                    'CT' + '\n' \
                                    'rows: ' + str(data_info['CT']['rows']) + '\n' \
                                    'columns: ' + str(data_info['CT']['cols']) + '\n' \
                                    'slices: ' + str(data_info['CT']['slices']) + '\n' \
                                    'slices thickness: ' + str(data_info['CT']['slice_thickness']) + ' mm' +'\n' \
                                    'pixel size: ' + str(data_info['CT']['pixel_size']) + ' mm' +'\n' \
                                    'device: ' + data_info['PT']['device']

            # sorting the dictionary
            temp_items = temp_data['PT'].items()
            temp_data['PT'] = sorted(temp_items)

            temp_items = temp_data['CT'].items()
            temp_data['CT'] = sorted(temp_items)

            # creation of 3D matrix as dictionary is not possible to be visualized in 2D from different views
            PT = np.zeros((data_info['PT']['rows'], data_info['PT']['cols'], data_info['PT']['slices']), dtype=int)
            i = 0
            for x in temp_data['PT']:
                PT[:,:,i] = x[1]
                i += 1

                    # additional filtration not available in Philips DICOM - skip that
            if ref_PT.get([0x0018,0x1210]) == None:
                ...
            else:
                data_info['PT']['filtration'] = ref_PT[0x0018,0x1210].value

                self.data_size['text'] ='PET' + '\n' \
                        'rows: ' + str(data_info['PT']['rows']) + '\n' \
                        'columns: ' + str(data_info['PT']['cols']) + '\n' \
                        'slices: ' + str(data_info['PT']['slices']) + '\n' \
                        'slices thickness: ' + str(data_info['PT']['slice_thickness']) + ' mm' +'\n' \
                        'pixel size: ' + str(data_info['PT']['pixel_size']) + ' mm' +'\n' \
                        'Post-recon filter FWHM: ' + data_info['PT']['filtration'] + ' mm' + '\n' \
                        'CT' + '\n' \
                        'rows: ' + str(data_info['CT']['rows']) + '\n' \
                        'columns: ' + str(data_info['CT']['cols']) + '\n' \
                        'slices: ' + str(data_info['CT']['slices']) + '\n' \
                        'slices thickness: ' + str(data_info['CT']['slice_thickness']) + ' mm' +'\n' \
                        'pixel size: ' + str(data_info['CT']['pixel_size']) + ' mm' +'\n' \
                        'device: ' + data_info['PT']['device']

            CT = np.zeros((data_info['CT']['rows'], data_info['CT']['cols'], data_info['CT']['slices']), dtype=int)
            i = 0
            for x in temp_data['CT']:
                CT[:,:,i] = x[1]
                i += 1

            # threshold the image to get rid of unwanted information - based on NEMA NU 2-2018
            PT_thresh = self.data_thresh(PT, int(self.threshold['PT'].get()))
            CT_thresh = self.data_thresh(CT, int(self.threshold['CT'].get()))

            # data stored in phantomData object
            self.phantom = phantomData(dirname, PT, CT, PT_thresh, CT_thresh, data_info)

            # obtain sorted sliced positions
            # testing point
            shift = 0 # shift in mm ... can be changed to test the correctness of calculations
            for i in temp_data['PT']: self.phantom.z_position_PT.append(i[0]+shift)
            for i in temp_data['CT']: self.phantom.z_position_CT.append(i[0])

            self.phantom.show_me_projections(self.phantom.PT_thresh, 'PT', [self.phantom_PT_cor, self.phantom_PT_sag, self.phantom_PT_trans],self.phantom.position, self.phantom.zoom)
            self.phantom.show_me_projections(self.phantom.CT_thresh, 'CT', [self.phantom_CT_cor, self.phantom_CT_sag, self.phantom_CT_trans],self.phantom.position, self.phantom.zoom)

            # =============================================================================
            # bind to mouse events
            # # =============================================================================
            self.phantom_PT_trans.bind("<Shift-MouseWheel>", self.mouse_zoom)
            self.phantom_CT_trans.bind("<Shift-MouseWheel>", self.mouse_zoom)

            self.phantom_PT_sag.bind("<Shift-MouseWheel>", self.mouse_zoom)
            self.phantom_CT_sag.bind("<Shift-MouseWheel>", self.mouse_zoom)

            self.phantom_PT_cor.bind("<Shift-MouseWheel>", self.mouse_zoom)
            self.phantom_CT_cor.bind("<Shift-MouseWheel>", self.mouse_zoom)

            self.phantom_PT_trans.bind("<ButtonPress-2>", self.mouse_click_reset)
            self.phantom_CT_trans.bind("<ButtonPress-2>", self.mouse_click_reset)

            self.phantom_PT_sag.bind("<ButtonPress-2>", self.mouse_click_reset)
            self.phantom_CT_sag.bind("<ButtonPress-2>", self.mouse_click_reset)

            self.phantom_PT_cor.bind("<ButtonPress-2>", self.mouse_click_reset)
            self.phantom_CT_cor.bind("<ButtonPress-2>", self.mouse_click_reset)
            # =============================================================================
            # data opened so additional frames can be switched on
            # =============================================================================
            frame_switch(self.frame_c, 1)
            frame_switch(self.frame_e, 1)

    # =============================================================================
    # mainWindow functions section
    # =============================================================================
    def calculate(self):
        ...
        # find the centre coordinates
        #%%
        self.phantom.pos_PT = self.phantom.line_finder(self.phantom.PT_thresh, int(self.masking.get()), int(self.sources.get()), int(self.skipper.get()))
        self.phantom.pos_CT = self.phantom.line_finder(self.phantom.CT_thresh, int(self.masking.get()), int(self.sources.get()), int(self.skipper.get()))

        if np.min(self.phantom.pos_PT) == 0 or np.min(self.phantom.pos_CT) == 0:
            tk.messagebox.showwarning("Warning", "Increase masking element size to catch the line source slope!")
        else:
            # result of the line_finder function is in the following format rows x cols x slices
            
            # image position patient DICOM tag stores information in the following order x (cols), y (rows), z (slices)
            # conversion cm_position from px to a real position in mm and shift according to image_position_patient

            px_size = float(self.phantom.all_setup['PT']['pixel_size'][0]), float(self.phantom.all_setup['PT']['pixel_size'][1])
            px_shift = float(self.phantom.all_setup['PT']['image_position_patient'][1]), float(self.phantom.all_setup['PT']['image_position_patient'][0])
            self.phantom.pos_PT_mm = self.phantom.pos_PT.copy()
            for i in self.phantom.pos_PT_mm:

                i[::2]= [(x*px_size[0])+px_shift[0] for x in i[::2]] # rows
                i[1::2]= [(x*px_size[1])+px_shift[1] for x in i[1::2]] # cols

            # conversion cm_position from px to a real position in mm
            px_size = float(self.phantom.all_setup['CT']['pixel_size'][0]), float(self.phantom.all_setup['CT']['pixel_size'][1])
            px_shift = float(self.phantom.all_setup['CT']['image_position_patient'][1]), float(self.phantom.all_setup['CT']['image_position_patient'][0])
            self.phantom.pos_CT_mm = self.phantom.pos_CT.copy()
            for i in self.phantom.pos_CT_mm:

                i[::2]= [(x*px_size[0])+px_shift[0] for x in i[::2]] # rows
                i[1::2]= [(x*px_size[1])+px_shift[1] for x in i[1::2]] # cols

            # calculation of the PT-CT position for x, y coordinates
            temp_diff_1 = np.subtract(self.phantom.pos_PT_mm, self.phantom.pos_CT_mm)

            # changing lines order in case that same line is not labeled in the same way on PT and CT first slice
            # if not handled correctly, algorith will follow wrong lines in subsequent slices
            myorder = [2,3,0,1]
            switched_line_pos = []
            switched_line_pos_mm = []
            for i in self.phantom.pos_CT_mm:
                i = [i[o] for o in myorder]
                switched_line_pos_mm.append(i)

            for i in self.phantom.pos_CT:
                i = [i[o] for o in myorder]
                switched_line_pos.append(i)

            temp_diff_2 = np.subtract(self.phantom.pos_PT_mm, switched_line_pos_mm)

            # check which positions give larger difference temp_diff_1 or temp_diff_2 ... the larger one is omited as the identified positions of PET and CT on different slices were different
            if np.max(np.abs(temp_diff_1)) > np.max(np.abs(temp_diff_2)):
                self.phantom.diffs = temp_diff_2
                self.phantom.pos_CT_mm = switched_line_pos_mm
                self.phantom.pos_CT = switched_line_pos
            else:
                self.phantom.diffs = temp_diff_1

            CE = [] # coregistration error
            x_shift = [] # shift in the x direction
            y_shift = [] # shift in the y direction
            for i in self.phantom.diffs:
                for x in list(range(0,len(i),2)):
                    CE.append(math.sqrt(i[x]**2+i[x+1]**2))
                    x_shift.append(i[x])
                    y_shift.append(i[x+1])
            # coregistration error - reshape to as many columns as sources available
            CE = np.reshape(CE, (np.size(self.phantom.diffs,0),int(np.size(self.phantom.diffs,1)/2)))
            # the max coregistration error
            index = np.where(CE == np.max(CE))

            # max shift in both directions
            x_shift_ = self.find_abs_max(x_shift)
            y_shift_ = self.find_abs_max(y_shift)

            # display the calculated values
            self.CE.set(str(np.round(np.max(CE),3))) # setting the tkinter labels accordingly
            self.xshift_max.set(str(np.round(x_shift_, 3)))
            self.xshift_mean.set(str(np.round(np.mean(x_shift),3)) + u"\u00B1" + str(np.round(np.std(x_shift),3)))
            self.yshift_max.set(str(np.round(y_shift_, 3)))
            self.yshift_mean.set(str(np.round(np.mean(y_shift),3)) + u"\u00B1" + str(np.round(np.std(y_shift),3)))

            row = index[0][0] # row in DICOM coordinates is the Y coordinate
            column = index[1][0] # column in DICOM coordinates is the X coordinate

            max_diff_y = self.phantom.diffs[row][2*column]
            max_diff_x = self.phantom.diffs[row][2*column+1]

            self.CE_xshift.set(str(np.round(np.max(max_diff_x),3))) # setting the tkinter label accordingly
            self.CE_yshift.set(str(np.round(np.max(max_diff_y),3))) # setting the tkinter label accordingly

            # intersection calculation
            self.phantom.intersection_PT = self.phantom.calculate_intersections(self.phantom.pos_PT, 'PT', int(self.skipper.get()))
            self.phantom.intersection_CT = self.phantom.calculate_intersections(self.phantom.pos_CT, 'CT', int(self.skipper.get()))

            zshift = np.round(0.5*(self.phantom.intersection_PT[0]-self.phantom.intersection_CT[0])+ \
                              0.5*(self.phantom.intersection_PT[1]-self.phantom.intersection_CT[1]),3)

            # display the calculated intercestion
            self.zshift.set(str(zshift))

    def find_abs_max(self, where):
        temp_max = np.max(where)
        temp_min = np.min(where)

        if abs(temp_min) > temp_max:
            return(temp_min)
        else:
            return(temp_max)


    def get_out_results(self):
        self.phantom.get_out_results(self.phantom.directory)

    def thresh_modif(self, i, modality):
        temp = int(self.threshold[modality].get())
        temp += i

        if temp <= 0: temp = 0
        elif temp >= 100: temp = 100

        self.threshold[modality].set(temp)

        if modality == 'PT':
            self.phantom.PT_thresh = self.data_thresh(self.phantom.PT, int(self.threshold['PT'].get()))
            self.phantom.show_me_projections(self.phantom.PT_thresh, 'PT', [self.phantom_PT_cor, self.phantom_PT_sag, self.phantom_PT_trans],self.phantom.position, self.phantom.zoom)

        if modality == 'CT':
            self.phantom.CT_thresh = self.data_thresh(self.phantom.CT, int(self.threshold['CT'].get()))
            self.phantom.show_me_projections(self.phantom.CT_thresh, 'CT', [self.phantom_CT_cor, self.phantom_CT_sag, self.phantom_CT_trans],self.phantom.position, self.phantom.zoom)

    def data_thresh(self, data, thresh):
        data_threshed = data.copy()
        data_threshed[data < np.max(data)*(0.01*thresh)] = 0

        return (data_threshed)

    def skipper_modif(self, i):
        temp = int(self.skipper.get())
        temp += i

        if temp <= 1: temp = 1
        elif temp >= 20: temp = 20

        self.skipper.set(temp)

    def mask_modif(self, i):
        temp = int(self.masking.get())
        temp += i

        if temp <= 3: temp = 3
        elif temp >= 100: temp = 100

        self.masking.set(temp)

    def mouse_zoom(self, event, *kwargs):
        zoom = [event.delta, event.x, event.y]
        self.phantom.slice_zoom(self, self.phantom.PT_thresh, zoom, 'PT', [self.phantom_PT_cor, self.phantom_PT_sag, self.phantom_PT_trans])
        self.phantom.slice_zoom(self, self.phantom.CT_thresh, zoom, 'CT', [self.phantom_CT_cor, self.phantom_CT_sag, self.phantom_CT_trans])


    # reset button
    def mouse_click_reset(self, event):
        # default setting of the phantom display -  self.phantom.show_me_slices(which, where, zoom)
        self.phantom.zoom = {'PT':1, 'CT':1}
        self.phantom.position = {'PT':[0,0], 'CT':[0,0]}

        self.phantom.show_me_projections(self.phantom.PT_thresh, 'PT', [self.phantom_PT_cor, self.phantom_PT_sag, self.phantom_PT_trans], self.phantom.position, self.phantom.zoom)
        self.phantom.show_me_projections(self.phantom.CT_thresh, 'CT', [self.phantom_CT_cor, self.phantom_CT_sag, self.phantom_CT_trans], self.phantom.position, self.phantom.zoom)

#%% global function
# function for switching frames on and off
def frame_switch(frame_name, on_off):
    if on_off == 0:
        for child in frame_name.winfo_children():
            child.configure(state='disabled')
    else:
        for child in frame_name.winfo_children():
            child.configure(state='normal')

root = tk.Tk()
alignment_analyzer = mainWindow(root)
root.mainloop()